@@ -5,7 +5,7 @@ use {
 };
 
 #[derive(Default)]
-pub(crate) struct LatestValidatorVotesForFrozenBanks {
+pub struct LatestValidatorVotesForFrozenBanks {
     // TODO: Clean outdated/unstaked pubkeys from this list.
     max_gossip_frozen_votes: HashMap<Pubkey, (Slot, Vec<Hash>)>,
     max_replay_frozen_votes: HashMap<Pubkey, (Slot, Vec<Hash>)>,
@@ -17,7 +17,7 @@ pub(crate) struct LatestValidatorVotesForFrozenBanks {
 impl LatestValidatorVotesForFrozenBanks {
     // `frozen_hash.is_some()` if the bank with slot == `vote_slot` is frozen
     // Returns whether the vote was actually added, and the latest voted frozen slot
-    pub(crate) fn check_add_vote(
+    pub fn check_add_vote(
         &mut self,
         vote_pubkey: Pubkey,
         vote_slot: Slot,
@@ -88,7 +88,7 @@ impl LatestValidatorVotesForFrozenBanks {
         )
     }
 
-    pub(crate) fn take_votes_dirty_set(&mut self, root: Slot) -> Vec<(Pubkey, SlotHashKey)> {
+    pub fn take_votes_dirty_set(&mut self, root: Slot) -> Vec<(Pubkey, SlotHashKey)> {
         let new_votes = std::mem::take(&mut self.fork_choice_dirty_set);
         new_votes
             .into_iter()
@@ -102,7 +102,7 @@ impl LatestValidatorVotesForFrozenBanks {
             .collect()
     }
 
-    pub(crate) fn max_gossip_frozen_votes(&self) -> &HashMap<Pubkey, (Slot, Vec<Hash>)> {
+    pub fn max_gossip_frozen_votes(&self) -> &HashMap<Pubkey, (Slot, Vec<Hash>)> {
         &self.max_gossip_frozen_votes
     }
 
