@@ -1,7 +1,6 @@
 use {
     crate::{cluster_info_vote_listener::VerifiedLabelVotePacketsReceiver, result::Result},
-    crossbeam_channel::Select,
-    solana_perf::packet::Packets,
+    solana_perf::packet::PacketBatch,
     solana_runtime::bank::Bank,
     solana_sdk::{
         account::from_account, clock::Slot, hash::Hash, pubkey::Pubkey, signature::Signature,
@@ -20,7 +19,7 @@ const MAX_VOTES_PER_VALIDATOR: usize = 1000;
 pub struct VerifiedVoteMetadata {
     pub vote_account_key: Pubkey,
     pub vote: Vote,
-    pub packet: Packets,
+    pub packet_batch: PacketBatch,
     pub signature: Signature,
 }
 
@@ -70,7 +69,7 @@ impl<'a> ValidatorGossipVotesIterator<'a> {
 ///
 /// Iterator is done after iterating through all vote accounts
 impl<'a> Iterator for ValidatorGossipVotesIterator<'a> {
-    type Item = Vec<Packets>;
+    type Item = Vec<PacketBatch>;
 
     fn next(&mut self) -> Option<Self::Item> {
         // TODO: Maybe prioritize by stake weight
@@ -116,7 +115,7 @@ impl<'a> Iterator for ValidatorGossipVotesIterator<'a> {
                                             None
                                         }
                                     })
-                                    .collect::<Vec<Packets>>()
+                                    .collect::<Vec<PacketBatch>>()
                             })
                         })
                 });
@@ -130,7 +129,7 @@ impl<'a> Iterator for ValidatorGossipVotesIterator<'a> {
     }
 }
 
-pub type SingleValidatorVotes = BTreeMap<(Slot, Hash), (Packets, Signature)>;
+pub type SingleValidatorVotes = BTreeMap<(Slot, Hash), (PacketBatch, Signature)>;
 
 #[derive(Default)]
 pub struct VerifiedVotePackets(HashMap<Pubkey, SingleValidatorVotes>);
@@ -141,16 +140,16 @@ impl VerifiedVotePackets {
         vote_packets_receiver: &VerifiedLabelVotePacketsReceiver,
         would_be_leader: bool,
     ) -> Result<()> {
-        let mut sel = Select::new();
-        sel.recv(vote_packets_receiver);
-        let _ = sel.ready_timeout(Duration::from_millis(200))?;
-        for gossip_votes in vote_packets_receiver.try_iter() {
+        const RECV_TIMEOUT: Duration = Duration::from_millis(200);
+        let vote_packets = vote_packets_receiver.recv_timeout(RECV_TIMEOUT)?;
+        let vote_packets = std::iter::once(vote_packets).chain(vote_packets_receiver.try_iter());
+        for gossip_votes in vote_packets {
             if would_be_leader {
                 for verfied_vote_metadata in gossip_votes {
                     let VerifiedVoteMetadata {
                         vote_account_key,
                         vote,
-                        packet,
+                        packet_batch,
                         signature,
                     } = verfied_vote_metadata;
                     if vote.slots.is_empty() {
@@ -161,7 +160,7 @@ impl VerifiedVotePackets {
                     let hash = vote.hash;
 
                     let validator_votes = self.0.entry(vote_account_key).or_default();
-                    validator_votes.insert((*slot, hash), (packet, signature));
+                    validator_votes.insert((*slot, hash), (packet_batch, signature));
 
                     if validator_votes.len() > MAX_VOTES_PER_VALIDATOR {
                         let smallest_key = validator_votes.keys().next().cloned().unwrap();
@@ -199,7 +198,7 @@ mod tests {
         s.send(vec![VerifiedVoteMetadata {
             vote_account_key,
             vote: vote.clone(),
-            packet: Packets::default(),
+            packet_batch: PacketBatch::default(),
             signature: Signature::new(&[1u8; 64]),
         }])
         .unwrap();
@@ -219,7 +218,7 @@ mod tests {
         s.send(vec![VerifiedVoteMetadata {
             vote_account_key,
             vote,
-            packet: Packets::default(),
+            packet_batch: PacketBatch::default(),
             signature: Signature::new(&[1u8; 64]),
         }])
         .unwrap();
@@ -241,7 +240,7 @@ mod tests {
         s.send(vec![VerifiedVoteMetadata {
             vote_account_key,
             vote,
-            packet: Packets::default(),
+            packet_batch: PacketBatch::default(),
             signature: Signature::new(&[1u8; 64]),
         }])
         .unwrap();
@@ -264,7 +263,7 @@ mod tests {
         s.send(vec![VerifiedVoteMetadata {
             vote_account_key,
             vote,
-            packet: Packets::default(),
+            packet_batch: PacketBatch::default(),
             signature: Signature::new(&[2u8; 64]),
         }])
         .unwrap();
@@ -283,7 +282,7 @@ mod tests {
         // No new messages, should time out
         assert_matches!(
             verified_vote_packets.receive_and_process_vote_packets(&r, true),
-            Err(Error::ReadyTimeout)
+            Err(Error::CrossbeamRecvTimeout(_))
         );
     }
 
@@ -303,7 +302,7 @@ mod tests {
             s.send(vec![VerifiedVoteMetadata {
                 vote_account_key,
                 vote,
-                packet: Packets::default(),
+                packet_batch: PacketBatch::default(),
                 signature: Signature::new(&[1u8; 64]),
             }])
             .unwrap();
@@ -340,7 +339,7 @@ mod tests {
             s.send(vec![VerifiedVoteMetadata {
                 vote_account_key,
                 vote,
-                packet: Packets::default(),
+                packet_batch: PacketBatch::default(),
                 signature: Signature::new_unique(),
             }])
             .unwrap();
@@ -394,7 +393,7 @@ mod tests {
                 s.send(vec![VerifiedVoteMetadata {
                     vote_account_key,
                     vote,
-                    packet: Packets::new(vec![Packet::default(); num_packets]),
+                    packet_batch: PacketBatch::new(vec![Packet::default(); num_packets]),
                     signature: Signature::new_unique(),
                 }])
                 .unwrap();
@@ -427,12 +426,12 @@ mod tests {
         // Get and verify batches
         let num_expected_batches = 2;
         for _ in 0..num_expected_batches {
-            let validator_batch: Vec<Packets> = gossip_votes_iterator.next().unwrap();
+            let validator_batch: Vec<PacketBatch> = gossip_votes_iterator.next().unwrap();
             assert_eq!(validator_batch.len(), slot_hashes.slot_hashes().len());
             let expected_len = validator_batch[0].packets.len();
             assert!(validator_batch
                 .iter()
-                .all(|p| p.packets.len() == expected_len));
+                .all(|batch| batch.packets.len() == expected_len));
         }
 
         // Should be empty now
@@ -461,7 +460,7 @@ mod tests {
         s.send(vec![VerifiedVoteMetadata {
             vote_account_key,
             vote,
-            packet: Packets::default(),
+            packet_batch: PacketBatch::default(),
             signature: Signature::new_unique(),
         }])
         .unwrap();
